#!BPY
#coding=utf-8
#
# BLENDED CITIES
#
# < 2.49 HOOK >

# a City creation script for Blender
#	This script allows you to create cities that
#	look fairly realistic from a closer and closer
#	distance, as the script evolves.
#	it relies on a core + a plugin and library system, that
#	allows to add new procedural building codes and premade
#	objects, that you can mix
#	together in the same city.
# 	documentation, logs, roadmap, download :
#	http://jerome.le.chat.free.fr

# ###################
# Alt-p to execute  #
# ###################

# author: Jerome Mahieux (littleneo) paris, france
# creation date: November 2008 (Blender 2.48a)
# derived from the 0.1 version of Suicidator City Engine by Arnaud Couturier (piiichan,nov. 2008,copyleft)

# CONTRIBUTORS
#
#	Kurt Whicher (another_noob)		(v0.2) naming convention, detailed building option in 0.2 GUI
#	Zsolt Fartek (b3d.hu)			hungarian translation
#	Thomas Krijnen (aothms)			Opensteer ! first sce plugin. plugin feature requests. emulation :-)
#	Angelo (bezel)					italian translation, code and en/us transl.
#	Dennis Davis(shaba1)			english documentation correction
# 	Philippe Roubal 				cars animation - collections of objects - first premade building !

# SPECIAL THANKS
#
#	Dale Best From Luma Arcade for sharing the club Silo mini-austin meshes and textures :
# 	www.lumaarcade.com
# 	**ClubSilo is an experimental non-commercial project by the studio.**

# GNU GENERAL PUBLIC LICENCE
#
#    This file is part of Blended Cities.
#
#    Blended Cities is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Blended Cities is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Blended Cities.  If not, see <http://www.gnu.org/licenses/>.
#
#Copyright 2009 Jerome Mahieux

"""
Name: 'Blended Cities'
Blender: 249
Group: 'Wizards'
"""

bcVersion='0.433'
lastChanges="28 november 2009"

import Blender
from Blender import *
from Blender.BGL import *
from Blender.Mathutils import *

import bpy, webbrowser, os,traceback,string,types,sys as Psys
from random import randint,random,uniform
from copy   import deepcopy
from math import *

# current path
path=sys.expandpath("//")
path=path.replace("\\", "/")
if path[-2:]=='//' : path=path[0:-1]
blendpath=path

# use preinstalled python or packed modules
pypath=path+'bin'+sys.sep+'python25mod.zip'
try :
	import shutil
	import xml.dom.minidom
except :
	print("(Python not found or missing modules : use packed 2.5 modules from /bin)")
	Psys.path.append(pypath)
	import shutil#, minidom

if pypath in Psys.path : PYTHON=False
else : PYTHON=True

def saveConfig(config='',update=False) :
	
	global conf
	
	# update
	if update :
		config=[config[0],config[1],conf['myPreset'],conf['myLanguage'],conf['refresh3dview'],conf['debugCore'],conf['debugToFile'],conf['beep'],conf['tips'],conf['removeLastGen']]

	# fresh install
	if len(config) == 2 :
		config.extend(['default','en',5,5,False,False,True,True])
	# updated from gui
	elif config=='' :
		config=[conf['BCpath'],conf['version'],presets[MSET[0].val],locales[LANG[0].val],R3DV[0].val,SDGL[0].val,bool(TDGO[0].val),bool(BEEP[0].val),bool(TIPS[0].val),bool(DELB[0].val)]

	conf = {
		'tooltips' : {
			'BCpath'		: 'path to Blended Cities directory',
			'version'		: 'Blended Cities version number',
			'myPreset'		: 'default city configuration',
			'myLanguage'	: 'GUI language',
			'refresh3dview'	: 'buildings number between each 3d view refresh (one object/building mode)',
			'debugCore'		: 'debug level',
			'debugToFile'	: '(WIP) write debug file in local temp',
			'beep'			: 'beeps on console warnings',
			'tips'			: 'display tips'
		},
		'limits' : {
			'debugCore'		: [1, 6],
			'refresh3dview'	: [0, 100]
		},
		'BCpath'			: config[0],
		'version'			: config[1],
		'myPreset'			: config[2],
		'myLanguage'		: config[3],
		'refresh3dview'		: config[4],
		'debugCore'			: config[5],
		'debugToFile'		: config[6],
		'beep'				: config[7],
		'tips'				: config[8],
		'removeLastGen'		: config[9],
	}

	Registry.SetKey('Blended_Cities', conf, True)			
				
def selectdir(newpath) :
	global path
	newpath=newpath.replace("\\", "/")
	dirs=newpath.split('/')
	newpath=''
	for d in range(0,len(dirs)-1) :	newpath +=dirs[d]+'/'
	path=checkpath(newpath)

def checkpath(path) :
	path=path.replace("\\", "/")
	if path[-2:]=='//' : path=path[0:-1]
	pathis=sys.exists(path)
	#print
	#print '->',path	
	#print 'file :',sys.basename(path)
	#print 'dir  :',sys.dirname(path)
	#print 'is   :',pathis
	if pathis <> 0 : 
		return path
	else :
		return False

def install(path) :
	
	def gui() :
		global bcVersion,path,conf,select
		width=Window.GetAreaSize()[0]
		height=Window.GetAreaSize()[1]
		y=height-20
		x=10
		w=width-20
		h=20
		sy=30
		Blender.BGL.glRasterPos2i(x,y)
		Draw.Text("Blended Cities Installation ("+bcVersion+")",'large')
		y -=sy
		error=0
		
		conf=Registry.GetKey('Blended_Cities', True) 
		if conf :

			try :
				savedversion=conf['version']
				savedpath=conf['BCpath']
			except :
				# wrong file or older format
				savedpath=0
	
		if path :
			# logo test
			#print path+'bin/img/logo_oce64x64.png'
			#print checkpath(path+'bin/img/logo_oce64x64.png')
			if checkpath(path+'bin/img/logo_oce64x64.png')  :
				# version test
				if checkpath(path+'bin/blended_cities_'+bcVersion.replace('.','')+'.py')  :
					
					Blender.BGL.glRasterPos2i(64+x,y)
					if conf :
						saveConfig([path,bcVersion],True)
						Draw.Text("Blended_Cities Config file updated")
					else :
						saveConfig([path,bcVersion])
						Draw.Text("Blended_Cities Config file created")
					image_logo=Image.Load(path+'bin/img/logo_oce64x64.png')
					
					y -=sy/2
					Blender.BGL.glRasterPos2i(64+x,y)
					Draw.Text("( .blender/scripts/bpydata/config )")
						
					y -=sy
					BGL.glEnable( BGL.GL_BLEND ) # alpha blending
					BGL.glBlendFunc(BGL.GL_SRC_ALPHA, BGL.GL_ONE_MINUS_SRC_ALPHA)
					Blender.Draw.Image(image_logo,x,y)
					BGL.glDisable( BGL.GL_BLEND )
					y -=sy
					Draw.PushButton('Done', 1, x, y, w, h, 'can we go now ?')
				else :
					error=2 # wrong bc version
			else : 
				error=1 # path exists but missing or wrong (logo test)
		else :
			error=1 # wrong path

		if error :

			if conf :
				if savedversion < bcVersion :
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text("Update from v"+savedversion+' to v'+bcVersion+" .")
					y -=sy
				elif savedversion > bcVersion :
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text("Downgrade from v"+savedversion+' to v'+bcVersion+" .")
					y -=sy/2
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text("if you don't want to downgrade, abort and")
					y -=sy/2
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text("have a look at the README file")
					y -=sy
			
			if error == 1 :
				if select :
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text("Wrong path.")
					y -=sy
				elif conf :
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text("cannot find the Blended Cities directory,")
					y -=sy/2
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text("the stored path is outdated.")
					y -=sy
			
			elif error == 2 :
			
				if select :
				
					Blender.BGL.glRasterPos2i(x,y)
					Draw.Text('blended_cities_'+bcVersion.replace('.','')+'.py is missing')
					if conf :
						y -=sy/2
						Blender.BGL.glRasterPos2i(x,y)
						Draw.Text('in this Blended Cities directory.')
						y -=sy/2
						Blender.BGL.glRasterPos2i(x,y)
						Draw.Text("you selected the directory of a previous version.")
						y -=sy/2
						Blender.BGL.glRasterPos2i(x,y)
						Draw.Text("(..else have a look at the README file :)")
						y -=sy
					else :
						Blender.BGL.glRasterPos2i(x,y)
						Draw.Text("something is wrong and unusual")
						y -=sy/2
						Blender.BGL.glRasterPos2i(x,y)
						Draw.Text("Have a look at the README file")
						y -=sy
			
			Blender.BGL.glRasterPos2i(x,y)
			Draw.Text("Please select the Blended Cities v"+bcVersion+" directory :")
			y -=sy
			Draw.PushButton('Select Dir', 2, x, y, w, h, 'browse to Blended Cities directory')
			
	def bevent(evt) :
		global select,path
		if evt==1 :
			Draw.Exit()
			main(path)
			return True
		if evt==2 :
			Window.FileSelector(selectdir, 'SELECTDIR')
			select=True
	def event(evt,val):
		if evt == Draw.ESCKEY or evt == Draw.QKEY :
			Draw.Exit()
			return False

	Draw.Register(gui, event, bevent)


def main(path) :
	global blendpath,bcVersion
	file = open(path+'bin/blended_cities_'+bcVersion.replace('.','')+'.py', "r")
	exec(file) in globals()
	file.close()

# begin

select=False
conf=Registry.GetKey('Blended_Cities', True) 

if conf :
	try :
		path = conf['BCpath']
		usedversion	= conf['version']
		if usedversion <> bcVersion : install(path)
		elif checkpath(path+'bin/blended_cities_'+bcVersion.replace('.','')+'.py')  :
			main(path)
		else : install(path)
	except :
		install(path)
else :
	# installation
	install(path) # new

